package example.localplugin.coolemall.recs;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import schedframe.events.scheduling.SchedulingEvent;
import schedframe.exceptions.ResourceException;
import schedframe.resources.CoolEmAllResourceType;
import schedframe.resources.computing.ComputingNode;
import schedframe.resources.computing.profiles.energy.airthroughput.StandardAirThroughputStateName;
import schedframe.scheduling.manager.resources.ClusterResourceManager;
import schedframe.scheduling.manager.resources.ResourceManager;
import schedframe.scheduling.manager.tasks.JobRegistry;
import schedframe.scheduling.plan.SchedulingPlanInterface;
import schedframe.scheduling.plan.impl.SchedulingPlan;
import schedframe.scheduling.plugin.grid.ModuleList;
import schedframe.scheduling.queue.TaskQueue;
import schedframe.scheduling.queue.TaskQueueList;
import schedframe.scheduling.tasks.TaskInterface;
import example.localplugin.BaseLocalSchedulingPlugin;
import gridsim.dcworms.DCWormsTags;

public class RECS_FCFS_Random_SP extends BaseLocalSchedulingPlugin {

	private Random rand;
	
	public RECS_FCFS_Random_SP () {
		rand = new Random(47);
	}

	public SchedulingPlanInterface<?> schedule(SchedulingEvent event, TaskQueueList queues, JobRegistry jobRegistry,
			 ResourceManager resManager, ModuleList modules) {

		ClusterResourceManager resourceManager = (ClusterResourceManager) resManager;
		SchedulingPlan plan = new SchedulingPlan();
		// choose the events types to serve.
		// Different actions for different events are possible.
		switch (event.getType()) {
		case START_TASK_EXECUTION:
		case TASK_FINISHED:
		//case TIMER:
			// our tasks are placed only in first queue (see
			// BaseLocalSchedulingPlugin.placeJobsInQueues() method)
			TaskQueue q = queues.get(0);
			
			List<ComputingNode> notSelectedNodes = null;
			try {
				notSelectedNodes = (List<ComputingNode>) resourceManager.getResourcesOfType(CoolEmAllResourceType.Node);
			} catch (ResourceException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
			// check all tasks in queue
			for (int i = 0; i < q.size(); i++) {
				TaskInterface<?> task = q.get(i);
				// if status of the tasks in READY
				if (task.getStatus() == DCWormsTags.READY) {
					ComputingNode node = chooseRandomProvider(resourceManager);
					if (node != null) {
						node.getAirThroughputInterface().setAirThroughputState(StandardAirThroughputStateName.FAN_ON);
						notSelectedNodes.remove(node);
						addToSchedulingPlan(plan, task, node.getName());
					}
				}
			}
			adjustOtherFans(notSelectedNodes);
			break;
		}
		return plan;
	}

	@SuppressWarnings("unchecked")
	private ComputingNode chooseRandomProvider(ClusterResourceManager resourceManager) {
		List<ComputingNode> nodes = null;
		try {
			nodes = (List<ComputingNode>) resourceManager.getResourcesOfType(CoolEmAllResourceType.Node);
		} catch (ResourceException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		nodes = filterNodes(nodes);
		int nodeIdx = rand.nextInt(nodes.size());
		return nodes.get(nodeIdx);
	}

	private List<ComputingNode> filterNodes(List<ComputingNode> nodes){
		List<ComputingNode> filteredNodes = new ArrayList<ComputingNode>();
		for (ComputingNode node : nodes) {
			if(node.getFreeProcessors().size() > 0)
				filteredNodes.add(node);
		}
		
		return filteredNodes;
	}
	private void adjustOtherFans(List<ComputingNode> nodes) {
		for (ComputingNode node : nodes) {
			if (node.getProcessors().size() == node.getFreeProcessorsNumber()) {
				node.getAirThroughputInterface().setAirThroughputState(StandardAirThroughputStateName.FAN_OFF);
			} else {
				node.getAirThroughputInterface().setAirThroughputState(StandardAirThroughputStateName.FAN_ON);
			}
		}
	}
}

